# Documentation

### Capabilities

Playwright is a library to automate [Chromium](https://www.chromium.org/Home), [Firefox](https://www.mozilla.org/en-US/firefox/new/) and [WebKit](https://webkit.org/) with a single API. Playwright is built to enable cross-browser web automation that is **ever-green**, **capable**, **reliable** and **fast**.

* Ever-green browser engines
* Headless execution
* Scenarios that span multiple pages, domains and iframes
* Auto-wait for elements to be ready before executing actions (like click, fill)
* Reliable signals instead of timeouts (like network response, popup, navigation)
* Intercept network activity for stubbing and mocking network requests
* Emulate mobile devices, geolocation, permissions
* Support for web components via shadow-piercing selectors
* Native input events for mouse and keyboard
* Upload and download files

### Table of contents

1. Introduction
    - [Why Playwright?](./why-playwright.md)
    - [Get started](./intro.md)
    - [Core concepts](./core-concepts.md)
    - [Debugging](./debug.md)
    - [Supported languages](./languages.md)
1. Guides
    - [Selectors](./selectors.md)
    - [Input](./input.md)
    - [Emulation](./emulation.md)
    - [Network](./network.md)
    - [Assertions](./assertions.md)
    - [Verification](./verification.md)
    - [Navigations](./navigations.md)
    - [Multi-page scenarios](./multi-pages.md)
1. Tutorials
    - [Authentication](./auth.md)
    - [Page object models](./pom.md)
1. Integrations
    - [Test runners](./test-runners.md)
    - [Docker](./docker/README.md)
    - [Continuous integration](./ci.md)
1. Reference
    - [API Reference](./api.md)
    - [Actionability](./actionability.md)
    - [Advanced installation](./installation.md)
    - [Extensibility](./extensibility.md)
1. Get help
    - [Slack community](https://join.slack.com/t/playwright/shared_invite/enQtOTEyMTUxMzgxMjIwLThjMDUxZmIyNTRiMTJjNjIyMzdmZDA3MTQxZWUwZTFjZjQwNGYxZGM5MzRmNzZlMWI5ZWUyOTkzMjE5Njg1NDg)
    - [Stack Overflow](https://stackoverflow.com/tags/playwright)
    - [GitHub](https://github.com/microsoft/playwright/issues)
